import React, { useEffect, useState } from 'react';
import axios from 'axios';
import PropTypes from 'prop-types';
import Table from 'react-bootstrap/Table';
import Alert from 'react-bootstrap/Alert';
import Badge from 'react-bootstrap/Badge';

import {readPbf} from '../utils/gtfs-rt-utils';
import TripUpdatesTableHead from './trip-updates-table-head';
import TripUpdatesTableEntries from './trip-updates-table-entries';
import config from '../config';
import {filterData} from '../utils/filter-data';

/*the simplest way to define a component is to write a JavaScript function*/
/*destructure props object*/
export default function TripUpdatesTable ({isFetched, oset, limit, filter}) {
    ////console.log('TripUpdatesTable: oset: ' + oset);
    //////console.log('TripUpdatesTable: limit: ' + limit);

    const [ary, setAry] = useState([]);
    const [aryFiltered, setAryFiltered] = useState([]);
    const [fetchCompleted, setFetchCompleted] = useState(isFetched);
    /*get arry in a JavaScript function*/
    const getRry = async () => {
        try {
            /*TODO handle errors: https://www.valentinog.com/blog/await-react/*/
            const address = `${config.API}gtfs-rt`;
            const res = await axios.get(address,
					{
					    responseType: 'arraybuffer'
					});

            if(res.data){
                ////console.log('getRry() res available');
		const rry = readPbf(res.data);
		const rryLngth= rry.length;
                ////console.log('getRry() rryLngh: ' + rryLngth);

		const rryETU = new Array();
		let countEntityTripUpdate = 0;
		rry.forEach(entity => {
		    const entityTripUpdate = entity.trip_update;
		    if (entityTripUpdate) {
			rryETU.push(entityTripUpdate);
			countEntityTripUpdate++;
		    }
		});
                ////console.log('getRry() countEntityTripUpdate: ' + countEntityTripUpdate);

		const rryOstLmt = new Array();
		let j = 0
		for(let i = (oset - 1) * limit; i < rryLngth ; i++){
		    if(j < limit){
			rryOstLmt.push(rryETU[i]);
			j++;
		    }
		}
		const rryOstLmtLngth = rryOstLmt.length;
                ////console.log('getRry() rryOstLmtLngh: ' + rryOstLmtLngth);
		
		setAry((ary) => rryOstLmt);
		//TODO let data=filterData(res.data,'tdb_stops',filter);
		//TODO setAryFiltered((aryFiltered) => data);
		setAryFiltered((aryFiltered) => rryOstLmt);
	    }else{
                console.error('getRry() res NOT available');
	    }
        } catch (err) {
            console.error('err.message: ' + err.message);
	    setAry((ary) => []);
	    setAryFiltered((aryFiltered) => []);
        }
    };
    useEffect(()=>{
	setAryFiltered((aryFiltered)=>{
	    let filtered=filterData(ary,'tdb_stops',filter);
	    return filtered;
	});
    },[filter]);
    useEffect(()=>{
	setFetchCompleted((fetchCompleted)=>true);
    }, [ary]);
    useEffect(() => {
	/*effect goes here*/
	getRry();
	/*use an empty dependency array to ensure the hook is running only once*/
	/*TODO study dependency array: https://reactjs.org/docs/hooks-effect.html*/
    }, [oset,limit]);
    if(fetchCompleted && aryFiltered.length > 0){
	/*return a React element*/
	return (
            <>
		<Table
		    striped
		    bordered
		    hover
		    size="sm"
		    variant="dark"
		    responsive
		>
                    <thead>
			<TripUpdatesTableHead />
		    </thead>
                    <tbody>
			<TripUpdatesTableEntries aryData={aryFiltered} />
		    </tbody>
		</Table>
            </>
	);
    }else{
	return (
	    <Alert variant={'secondary'} onClose={() => setShow(false)} dismissible>
		<Badge bg="secondary">TripUpdate</Badge> entities loading...
            </Alert>

	);
    }
}
TripUpdatesTable.propTypes = {
    isFetched: PropTypes.bool,
    offset: PropTypes.number,
    limit: PropTypes.number,
    filter: PropTypes.string
};
